import API from 'immutable-cursors/lib/API';
import pathToSeq from 'immutable-cursors/lib/pathToSeq';
import TreeCursor from './TreeCursor';
import TreeUtils from './TreeUtils';


export default class TreeAPI extends API {

	constructor(treeUtils) {
		super();
		this.treeUtils = treeUtils || new TreeUtils();
	}

	treeFrom(rootData, keyPath, onChange) {
		let path = pathToSeq(keyPath);
		return this.makeCursor(rootData, path, onChange, undefined, TreeCursor, {
			rootPath: path
		});
	}

	makeTreeCursor(cursor, keyPathOrList) {
		if (Array.isArray(keyPathOrList)) {
			return keyPathOrList.map(keyPath => {
				return this.makeCursor(
					cursor._rootData,
					keyPath,
					cursor._onChange,
					undefined,
					TreeCursor,
					cursor._sharedOptions
				);
			});
		} else {
			return this.makeCursor(
				cursor._rootData,
				keyPathOrList,
				cursor._onChange,
				undefined,
				TreeCursor,
				cursor._sharedOptions
			);
		}
	}

	export() {
		let api = super.export();
		api.treeFrom = this.treeFrom.bind(this);
		return api;
	}
}
