import KeyedCursor from 'immutable-cursors/lib/KeyedCursor';
import { Seq } from 'immutable';

class TreeCursor extends KeyedCursor {

	constructor(...args) {
		super(...args);
		this._rootPath = this._sharedOptions.rootPath;
		this._treeUtils = this._api.treeUtils;
	}

	_callOnRoot(fn, ...args) {
		let result = fn.call(this._treeUtils, this._rootData, this._rootPath, ...args);
		if (Seq.isSeq(result) || Array.isArray(result)) {
			return this._api.makeTreeCursor(this, result);
		}
		return result;
	}

	_callOnThis(fn, ...args) {
		let result = fn.call(this._treeUtils, this._rootData, this._keyPath, ...args);
		if (Seq.isSeq(result) || Array.isArray(result)) {
			return this._api.makeTreeCursor(this, result);
		}
		return result;
	}

	id() {
		return this._callOnThis(this._treeUtils.id);
	}

	type() {
		return this._callOnThis(this._treeUtils.type);
	}

	root() {
		return this._api.makeTreeCursor(
			this,
			this._rootPath
		);
	}

	findInTree(comparator) {
		return this._callOnThis(this._treeUtils.findInTree, comparator);
	}

	filterInTree(comparator) {
		return this._callOnThis(this._treeUtils.filterInTree, comparator);
	}

	getById(id) {
		return this._callOnThis(this._treeUtils.getById, id);
	}

	getByType(...types) {
		return this._callOnThis(this._treeUtils.getByType, ...types);
	}

	firstChild() {
		return this._callOnRoot(this._treeUtils.firstChild, this._keyPath);
	}

	lastChild() {
		return this._callOnRoot(this._treeUtils.lastChild, this._keyPath);
	}

	siblings() {
		return this._callOnRoot(this._treeUtils.siblings, this._keyPath);
	}

	previousSibling() {
		return this._callOnRoot(this._treeUtils.previousSibling, this._keyPath);
	}

	nextSibling() {
		return this._callOnRoot(this._treeUtils.nextSibling, this._keyPath);
	}

	parent() {
		return this._callOnRoot(this._treeUtils.parent, this._keyPath);
	}

	ancestors() {
		return this._callOnRoot(this._treeUtils.ancestors, this._keyPath);
	}

	descendants() {
		return this._callOnRoot(this._treeUtils.descendants, this._keyPath);
	}

	childIndex() {
		return this._callOnRoot(this._treeUtils.childIndex, this._keyPath);
	}

	childNodes() {
		return this._callOnRoot(this._treeUtils.childNodes, this._keyPath);
	}

	childAt(index) {
		return this._callOnRoot(this._treeUtils.childAt, this._keyPath, index);
	}

	numChildNodes() {
		return this._callOnRoot(this._treeUtils.numChildNodes, this._keyPath);
	}

	hasChildNodes() {
		return this._callOnRoot(this._treeUtils.hasChildNodes, this._keyPath);
	}

	right() {
		return this._callOnRoot(this._treeUtils.right, this._keyPath);
	}

	left() {
		return this._callOnRoot(this._treeUtils.left, this._keyPath);
	}

	firstDescendant() {
		return this._callOnRoot(this._treeUtils.firstDescendant, this._keyPath);
	}

	lastDescendant() {
		return this._callOnRoot(this._treeUtils.lastDescendant, this._keyPath);
	}
}

export default TreeCursor;
